# Generated by Django 5.2.4 on 2025-10-20 13:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('history4feed', '0005_feed_datetime_modified'),
    ]

    operations = [
        migrations.AddField(
            model_name='job',
            name='completion_time',
            field=models.DateTimeField(default=None, null=True),
        ),
    ]
