# Generated by Django 5.1.6 on 2025-05-02 13:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('history4feed', '0003_alter_feed_description'),
    ]

    operations = [
        migrations.AlterField(
            model_name='fulltextjob',
            name='status',
            field=models.CharField(choices=[('retrieved', 'Retrieved'), ('skipped', 'Skipped'), ('cancelled', 'Cancelled'), ('failed', 'Failed'), ('retrieving', 'Retrieving')], default='retrieving', max_length=15),
        ),
        migrations.AlterField(
            model_name='job',
            name='state',
            field=models.CharField(choices=[('pending', 'Pending'), ('running', 'Running'), ('success', 'Success'), ('cancelled', 'Cancelled'), ('failed', 'Failed')], default='pending', help_text='state of the job', max_length=12),
        ),
    ]
